import { ClientSideBaseVisitor } from 'graphql-codegen-visitor-plugin-common';
import * as autoBind from 'auto-bind';
import { parse, print } from 'graphql';
const R_MOD = /module\:\s*"([^"]+)"/; // matches: module: "..."
const R_NAME = /name\:\s*"([^"]+)"/; // matches: name: "..."
function R_DEF(directive) {
    return new RegExp(`\\s+\\@${directive}\\([^)]+\\)`, 'gm');
}
export class ApolloAngularVisitor extends ClientSideBaseVisitor {
    constructor(fragments, _allOperations, rawConfig) {
        super(fragments, rawConfig, {});
        this._allOperations = _allOperations;
        autoBind(this);
    }
    getImports() {
        const baseImports = super.getImports();
        const imports = [`import { Injectable } from '@angular/core';`, `import Apollo from 'apollo-angular';`];
        const defs = {};
        this._allOperations
            .filter(op => this._operationHasDirective(op, 'NgModule'))
            .forEach(op => {
            const def = this._extractNgModule(op);
            // by setting key as link we easily get rid of duplicated imports
            // every path should be relative to the output file
            defs[def.link] = {
                path: def.path,
                module: def.module
            };
        });
        Object.keys(defs).forEach(key => {
            const def = defs[key];
            // Every Angular Module that I've seen in my entire life use named exports
            imports.push(`import { ${def.module} } from '${def.path}';`);
        });
        return [baseImports, ...imports].join('\n');
    }
    _extractNgModule(operation) {
        const [, link] = print(operation).match(R_MOD);
        const [path, module] = link.split('#');
        return {
            path,
            module,
            link
        };
    }
    _operationHasDirective(operation, directive) {
        return print(operation).includes(`@${directive}`);
    }
    _removeDirective(document, directive) {
        if (this._operationHasDirective(document, directive)) {
            return parse(print(document).replace(R_DEF(directive), '')).definitions[0];
        }
        return parse(print(document)).definitions[0];
    }
    _removeDirectives(document, directives) {
        return directives.reduce((doc, directive) => this._removeDirective(doc, directive), document);
    }
    _extractDirective(operation, directive) {
        const directives = print(operation).match(R_DEF(directive));
        if (directives.length > 1) {
            throw new Error(`The ${directive} directive used multiple times in '${operation.name}' operation`);
        }
        return directives[0];
    }
    _prepareDocument(documentStr) {
        return print(this._removeDirectives(parse(documentStr).definitions[0], ['NgModule', 'namedClient']));
    }
    _namedClient(operation) {
        if (!this._operationHasDirective(operation, 'namedClient')) {
            return '';
        }
        const name = this._extractNamedClient(operation);
        return `client = '${name}';`;
    }
    // tries to find namedClient directive and extract {name}
    _extractNamedClient(operation) {
        const [, name] = this._extractDirective(operation, 'namedClient').match(R_NAME);
        return name;
    }
    _providedIn(operation) {
        if (this._operationHasDirective(operation, 'NgModule')) {
            return this._extractNgModule(operation).module;
        }
        return `'root'`;
    }
    buildOperation(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        const content = `
  @Injectable({
    providedIn: ${this._providedIn(node)}
  })
  export class ${this.convertName(node.name.value)}GQL extends Apollo.${operationType}<${operationResultType}, ${operationVariablesTypes}> {
    document = ${documentVariableName};
    ${this._namedClient(node)}
  }`;
        return content;
    }
}
//# sourceMappingURL=visitor.js.map